/**
 * @license
 * Qwik Speak
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/robisim74/qwik-speak/blob/main/LICENSE
 */
import { isDev } from "@builder.io/qwik/build";
import { getLang, getSpeakContext } from "./context.qwik.mjs";
import { logWarn } from "./log.qwik.mjs";
const localizePath = () => {
  const { config } = getSpeakContext();
  const currentLang = getLang();
  const getRegEpx = (lang) => new RegExp(`(/${lang}/)|(/${lang}$)|(/(${lang})(?=\\?))`);
  const replace = (pathname, lang) => {
    const langParam = config.supportedLocales.find((locale) => getRegEpx(locale.lang)?.test(pathname))?.lang;
    if (langParam) {
      if (lang !== config.defaultLocale.lang)
        pathname = pathname.replace(langParam, lang);
      else
        pathname = pathname.replace(getRegEpx(langParam), "/");
    } else if (lang !== config.defaultLocale.lang)
      pathname = `/${lang}${pathname}`;
    if (!isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed" && isDefaultDomain(lang))
        pathname = pathname.replace(getRegEpx(lang), "/");
    }
    return pathname;
  };
  const localize = (route, lang) => {
    lang ?? (lang = currentLang);
    if (Array.isArray(route))
      return route.map((path) => replace(path, lang));
    if (typeof route === "string")
      return replace(route, lang);
    let url = new URL(route);
    if (config.domainBasedRouting)
      url = localizeDomain(url, lang);
    url.pathname = replace(url.pathname, lang);
    return url.toString().replace(/\/\?/, "?");
  };
  return localize;
};
const translatePath = () => {
  const { config } = getSpeakContext();
  const rewriteRoutes = config.rewriteRoutes;
  const currentLang = getLang();
  const normalizePath = (pathname) => {
    let source = rewriteRoutes?.find((rewrite) => pathname === `/${rewrite.prefix}` || pathname.startsWith(`/${rewrite.prefix}/`) || pathname.startsWith(`${rewrite.prefix}/`));
    if (source)
      pathname = pathname === `/${source.prefix}` ? "/" : pathname.startsWith("/") ? pathname.substring(`/${source.prefix}`.length) : pathname.substring(`${source.prefix}/`.length);
    if (!isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed") {
        if (!source)
          source = rewriteRoutes?.find((rewrite) => rewrite.lang == currentLang);
      }
    }
    if (source) {
      const sourceEntries = Object.entries(source.paths).map(([from, to]) => [
        to,
        from
      ]);
      const revertedPaths = Object.fromEntries(sourceEntries);
      const splitted = pathname.split("/");
      const translated = splitted.map((part) => revertedPaths[part] ?? part);
      return translated.join("/");
    }
    return pathname;
  };
  const rewritePath = (pathname, prefix) => {
    let splitted = pathname.split("/");
    let destination = rewriteRoutes?.find((rewrite) => rewrite.prefix === prefix);
    if (prefix && destination) {
      const keys = Object.keys(destination.paths);
      const translating = splitted.some((part) => keys.includes(part));
      if (pathname === "/" || translating)
        pathname = pathname.startsWith("/") ? `/${prefix}` + pathname : `${prefix}/` + pathname;
    }
    if (!isDev) {
      if (config.domainBasedRouting?.prefix === "as-needed") {
        if (prefix && !destination)
          destination = rewriteRoutes?.find((rewrite) => rewrite.lang === prefix);
      }
    }
    splitted = pathname.split("/");
    const translated = splitted.map((part) => destination?.paths[part] ?? part);
    return translated.join("/");
  };
  const slashPath = (pathname, rewrote) => {
    if (pathname.endsWith("/") && !rewrote.endsWith("/"))
      return rewrote + "/";
    else if (!pathname.endsWith("/") && rewrote.endsWith("/") && rewrote !== "/")
      return rewrote.substring(0, rewrote.length - 1);
    return rewrote;
  };
  const translateOne = (pathname, lang) => {
    const normalized = normalizePath(pathname);
    const rewrote = rewritePath(normalized, lang);
    return slashPath(pathname, rewrote);
  };
  const translate = (route, lang) => {
    lang ?? (lang = currentLang);
    if (!rewriteRoutes) {
      if (isDev)
        logWarn(`translatePath: rewriteRoutes not found`);
      return route;
    }
    if (Array.isArray(route))
      return route.map((path) => translateOne(path, lang));
    if (typeof route === "string")
      return translateOne(route, lang);
    let url = new URL(route);
    if (config.domainBasedRouting)
      url = translateDomain(url, lang);
    url.pathname = translateOne(url.pathname, lang);
    return url.toString();
  };
  return translate;
};
const extractFromDomain = (route, domains) => {
  const hostname = !route.hostname.startsWith("localhost") ? route.hostname : route.host;
  const domain = domains.find((value) => value.domain === hostname);
  return domain && ("lang" in domain ? domain.lang : "prefix" in domain ? domain.prefix : void 0);
};
const extractFromUrl = (route) => {
  const parts = route.pathname.split("/");
  return route.pathname.startsWith("/") ? parts[1] : parts[0];
};
const validateLocale = (lang) => {
  return /^([a-z]{2,3})(-[A-Z][a-z]{3})?(-[A-Z]{2})?$/.test(lang);
};
function toPrefixAsNeeded(rewriteRoutes) {
  if (isDev)
    return rewriteRoutes;
  const routes = rewriteRoutes.map((rewrite) => ({
    prefix: rewrite.domain ? void 0 : rewrite.prefix,
    paths: rewrite.paths,
    lang: rewrite.prefix,
    domain: rewrite.domain,
    withDomain: rewrite.withDomain
  }));
  return routes;
}
const localizeDomain = (url, lang) => {
  const { config } = getSpeakContext();
  const locale = config.supportedLocales.find((value) => value.lang === lang);
  const domain = locale?.domain || locale?.withDomain;
  if (!domain) {
    if (isDev)
      logWarn(`localizeDomain: domain not found`);
  } else if (!isDev) {
    if (!domain.startsWith("localhost"))
      url.hostname = domain;
    else
      url.host = domain;
  }
  return url;
};
const translateDomain = (url, lang) => {
  const { config } = getSpeakContext();
  const rewriteRoutes = config.rewriteRoutes;
  const rewrite = rewriteRoutes?.find((value) => value.lang === lang || value.prefix === lang);
  const domain = rewrite?.domain || rewrite?.withDomain || // Default locale
  rewriteRoutes.find((rewrite2) => rewrite2.domain && Object.keys(rewrite2.paths).length === 0)?.domain;
  if (!domain) {
    if (isDev)
      logWarn(`translateDomain: domain not found`);
  } else if (!isDev) {
    if (!domain.startsWith("localhost"))
      url.hostname = domain;
    else
      url.host = domain;
  }
  return url;
};
const isDefaultDomain = (lang) => {
  const { config } = getSpeakContext();
  return config.supportedLocales.find((value) => value.lang === lang)?.domain !== void 0;
};
export {
  extractFromDomain,
  extractFromUrl,
  localizePath,
  toPrefixAsNeeded,
  translatePath,
  validateLocale
};
