/**
 * @license
 * Qwik Speak
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/robisim74/qwik-speak/blob/main/LICENSE
 */
import { getLocale, inlinedQrl, useContextProvider, useTaskQrl, useLexicalScope, useOnWindow, componentQrl, _jsxC, Slot } from "@builder.io/qwik";
import { isDev, isServer } from "@builder.io/qwik/build";
import { setSpeakServerContext, SpeakContext, setGetLangFn, setSpeakClientContext, getSpeakContext } from "./context.qwik.mjs";
import { loadTranslations } from "./load.qwik.mjs";
import { logWarn, logDebug, logDebugInline } from "./log.qwik.mjs";
const useQwikSpeak = (props) => {
  const lang = getLocale("");
  const resolvedConfig = {
    rewriteRoutes: props.config.rewriteRoutes,
    defaultLocale: props.config.defaultLocale,
    supportedLocales: props.config.supportedLocales,
    assets: props.config.assets,
    runtimeAssets: props.config.runtimeAssets,
    keySeparator: props.config.keySeparator || ".",
    keyValueSeparator: props.config.keyValueSeparator || "@@",
    domainBasedRouting: props.config.domainBasedRouting,
    showDebugMessagesLocally: props.config.showDebugMessagesLocally ?? true
  };
  let resolvedLocale = resolvedConfig.supportedLocales.find((value) => value.lang === lang);
  if (!resolvedLocale) {
    resolvedLocale = resolvedConfig.defaultLocale;
    if (isDev)
      logWarn(`Locale not resolved. Fallback to default locale: ${resolvedConfig.defaultLocale.lang}`);
  } else if (isDev)
    logDebug(resolvedConfig.showDebugMessagesLocally, `Resolved locale: ${resolvedLocale.lang}`);
  if (props.currency)
    resolvedLocale.currency = props.currency;
  if (props.timeZone)
    resolvedLocale.timeZone = props.timeZone;
  const resolvedTranslationFn = {
    loadTranslation$: props.translationFn?.loadTranslation$ ?? /* @__PURE__ */ inlinedQrl(() => null, "useQwikSpeak_resolvedTranslationFn_T9Fy9UxepgA")
  };
  const state = {
    locale: Object.assign({}, resolvedLocale),
    translation: Object.fromEntries(resolvedConfig.supportedLocales.map((value) => [
      value.lang,
      {}
    ])),
    config: Object.assign({}, resolvedConfig),
    translationFn: resolvedTranslationFn
  };
  const { config } = state;
  setSpeakServerContext(config);
  setGetLangFn(() => getLocale(config.defaultLocale.lang));
  useContextProvider(SpeakContext, state);
  useTaskQrl(/* @__PURE__ */ inlinedQrl(async () => {
    const [config2, props2, state2] = useLexicalScope();
    if (isServer)
      await loadTranslations(state2, config2.assets, config2.runtimeAssets, props2.langs);
  }, "useQwikSpeak_useTask_0fwIeNJbECQ", [
    config,
    props,
    state
  ]));
  const resumeContext$ = /* @__PURE__ */ inlinedQrl(() => {
    const [resolvedConfig2, state2] = useLexicalScope();
    const { locale, translation, config: config2 } = state2;
    setSpeakClientContext(locale, translation, config2);
    setGetLangFn(() => locale.lang);
    if (isDev) {
      const _speakContext = getSpeakContext();
      logDebugInline(resolvedConfig2.showDebugMessagesLocally, "Client context", _speakContext);
    }
    if (isDev) {
      if (import.meta.hot) {
        import.meta.hot.send("qwik-speak:lang", {
          msg: locale.lang
        });
        logDebugInline(resolvedConfig2.showDebugMessagesLocally, "Ready");
      }
    }
  }, "useQwikSpeak_resumeContext_6bH92HveK8o", [
    resolvedConfig,
    state
  ]);
  useOnWindow("load", resumeContext$);
};
const QwikSpeakMockProvider = /* @__PURE__ */ componentQrl(/* @__PURE__ */ inlinedQrl((props) => {
  const lang = props.locale?.lang;
  const resolvedConfig = {
    rewriteRoutes: props.config.rewriteRoutes,
    defaultLocale: props.config.defaultLocale,
    supportedLocales: props.config.supportedLocales,
    assets: props.config.assets,
    runtimeAssets: props.config.runtimeAssets,
    keySeparator: props.config.keySeparator || ".",
    keyValueSeparator: props.config.keyValueSeparator || "@@",
    domainBasedRouting: props.config.domainBasedRouting,
    showDebugMessagesLocally: props.config.showDebugMessagesLocally ?? true
  };
  let resolvedLocale = resolvedConfig.supportedLocales.find((value) => value.lang === lang);
  if (!resolvedLocale)
    resolvedLocale = resolvedConfig.defaultLocale;
  const resolvedTranslationFn = {
    loadTranslation$: props.translationFn?.loadTranslation$ ?? /* @__PURE__ */ inlinedQrl(() => null, "QwikSpeakMockProvider_component_resolvedTranslationFn_aOSQ00KYdkc")
  };
  const state = {
    locale: Object.assign({}, resolvedLocale),
    translation: Object.fromEntries(resolvedConfig.supportedLocales.map((value) => [
      value.lang,
      {}
    ])),
    config: Object.assign({}, resolvedConfig),
    translationFn: resolvedTranslationFn
  };
  const { locale, config } = state;
  const { config: _config } = getSpeakContext();
  Object.assign(_config, config);
  setGetLangFn(() => locale.lang);
  useContextProvider(SpeakContext, state);
  useTaskQrl(/* @__PURE__ */ inlinedQrl(async () => {
    const [config2, props2, state2] = useLexicalScope();
    await loadTranslations(state2, config2.assets, config2.runtimeAssets, props2.langs);
  }, "QwikSpeakMockProvider_component_useTask_0eOz05v4yM0", [
    config,
    props,
    state
  ]));
  return /* @__PURE__ */ _jsxC(Slot, null, 3, "6l_0");
}, "QwikSpeakMockProvider_component_zQJA0JbOhj0"));
export {
  QwikSpeakMockProvider,
  useQwikSpeak
};
